const { Reader } = require("@maxmind/geoip2-node");
const fs = require("fs");
import * as path from "path";

let geoReader: any;

const localDbName = "GeoLite2-Country.mmdb";
const remoteDbName = "RemoteGeoLite2-Country.mmdb";

const downloadDb = async () => {
  if (!process.env.GEO_DB_URL) {
    console.log("No GEO_DB_URL environment variable found, using local file");
    return;
  }
  const https = require("https");
    const file = fs.createWriteStream(path.join(__dirname, remoteDbName));
  await new Promise((resolve: any, reject: any) => {
    https.get(process.env.GEO_DB_URL, (response: any) => {
      response.pipe(file);
      file.on("finish", () => {
        file.close();
        console.log("Finished downloading GEO DB");
        console.log(`File size: ${fs.statSync(path.join(__dirname, remoteDbName)).size}`)
        resolve();
      });
    }).on("error", (err: any) => {
      fs.unlink(path.join(__dirname, remoteDbName));
      reject(err.message);
    });
  });
};

const loadReader = async (): Promise<any> => {
  if(geoReader) { return geoReader; }

  await downloadDb();
  const remoteDbPath = path.join(__dirname, remoteDbName);
  if (fs.existsSync(remoteDbPath)) {
    geoReader = await Reader.open(remoteDbPath);
    return geoReader;
  }

  const dbPath = path.join(__dirname, localDbName);
  geoReader = await Reader.open(dbPath);
  return geoReader;
}

const getCountry = async (ip: string) => {
  const reader = await loadReader();
  return reader.country(ip);
}

const handler = async (req: any, res: any) => {

  const forwardedFor = req.headers["x-forwarded-for"];
  const ips = forwardedFor ? forwardedFor.split(",") : [];
  const ip = ips[0] || req.connection.remoteAddress;

  try {
    const geo = await getCountry(ip) as any;
    res.status(200).json({
      lang: geo.country.names.en === "Vietnam" ? "vi" : "en",
      info: { ...{ip} }
    });
  } catch(err) {
    console.error(err);
    res.status(200).json({
      lang: "en",
      info: { ...{ip}, err }
    });
  }
}

export { getCountry, handler };
